#include "binary_c_python.h"
//#include "binary_c_api.h"
#include "binary_c_API.h"
#include "binary_c_API_prototypes.h"
#include <time.h>
#include <sys/timeb.h>
#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>


/*
 * apitest
 *
 * Short test programme to throw random binary systems at binary_c's
 * library via its API.
 *
 * Note that it looks more complicated than it is because I have included
 * code to capture binary_c's stdout stream and output it here.
 *
 * This code sends output to stderr : you should use apitest.sh to run it
 * and hence force output to your terminal's stdout.
 *
 * Output lines:
 *
 * APITEST ....  is information about what this code is doing.
 * STATUS  ....  is information about the binary system.
 * BINARY_C .... is output from binary_c (see iterate_logging.c etc.)
 *               which would have gone to stdout
 *
 * If you define the NO_OUTPUT macro, there will be no output except
 * the memory allocation and test system information. This is useful for speed tests,
 * but note that you may end up in a race condition where the pipe which replaces
 * stdout's buffer fills and hence the code stops.
 *
 * Note:
 * I have tested this with gcc 4.7.2 (Ubuntu 12.10) only.
 */


// #define _CAPTURE
#ifdef _CAPTURE
static void show_stdout(void);
static void capture_stdout(void);
#endif


/* global variables */
int out_pipe[2];
int stdoutwas;

int main(int argc,
         char * argv[])
{
    char * argstring = MALLOC(sizeof(char) * (size_t)STRING_LENGTH);
    snprintf(argstring,
             STRING_LENGTH,
             "binary_c M_1 %g M_2 %g separation %g orbital_period %g metallicity %g max_evolution_time %g\n",
             20.0,
             15.0,
             0.0,
             3.0,
             0.02,
             15000.0);

    char * buffer ;
    size_t nbytes;
    int out = run_binary(argstring,
                     &buffer,
                     &nbytes);

    printf("output (binary_c returned %d)\n%s\n",out,buffer);

    free(buffer);
    
    return out;
}


int run_binary(char * argstring,
               char ** buffer,
               size_t * nbytes)
{
    /* memory for N binary systems */
    struct libbinary_c_stardata_t *stardata;
    struct libbinary_c_store_t * store = NULL;

    printf("argstring : %s\n",argstring);
    fflush(stdout);

    stardata = NULL;
    binary_c_new_system(&stardata,
                        NULL,
                        NULL,
                        &store,
                        &argstring,
                        -1);
    snprintf(stardata->preferences->log_filename,
             STRING_LENGTH-1,
             "%s",
             "/dev/null");
    snprintf(stardata->preferences->api_log_filename_prefix,
             STRING_LENGTH-1,
             "%s",
             "/dev/null");
    stardata->preferences->internal_buffering = 2;
    stardata->preferences->batchmode = BATCHMODE_LIBRARY;

    binary_c_evolve_for_dt(stardata,
                           stardata->model.max_evolution_time);

    /*
     * Save the buffer pointer
     */
    binary_c_buffer_info(stardata,&buffer,nbytes);

    /*
     * And free everything else
     */    
    binary_c_free_memory(&stardata,TRUE,TRUE,FALSE);
    binary_c_free_store_contents(store);
    
    return 0;
}
